// apnd.cpp : Defines the entry point for the console application.
//
#define LINUX_PLATFORM
#define noWINDOWS_PLATFORM
#define noEMULATE

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h> 
#include <math.h> 
#include <time.h>
#include <unistd.h>
 
#if defined(WINDOWS_PLATFORM)
#include <windows.h>
#elif defined(LINUX_PLATFORM)
#include <dlfcn.h>
#endif

#define Val(a) atoi(a)
#define TChr char

#include "btchannel.h"
#include "TomTom.h"

// easier to read ...
#define	PHONENETWORK_ERROR_NONE			TomTom_PHONENETWORK_ERROR_NONE
#define	PHONENETWORK_ERROR_GENERAL	TomTom_PHONENETWORK_ERROR_GENERAL
#define	PHONENETWORK_ERROR_FILE			TomTom_PHONENETWORK_ERROR_FILE
#define	PHONENETWORK_ERROR_MEMORY		TomTom_PHONENETWORK_ERROR_MEMORY
#define	PHONENETWORK_ERROR_NODATA		TomTom_PHONENETWORK_ERROR_NODATA
#define	PHONENETWORK_MAX_SIZE				TomTom_PHONENETWORK_MAX_SIZE

// Function typedefs
typedef int		(*PNDPROC_INITIALISE)(const char*	, TomTom_PNDPROC_RECVDATA , TomTom_PNDPROC_SENDDATA );
typedef void	(*PNDPROC_DEINITIALISE)(int );
typedef int  	(*PNDPROC_GETVERSIONLIBRARY)(int );
typedef int	  (*PNDPROC_GETVERSIONDATABASE)(int );

typedef int		(*PNDPROC_DETECTPHONEANDNETWORK)(int ); 
typedef int   (*PNDPROC_GETDETECTEDPHONE)(int , char *, int , char *, int ); 
typedef int   (*PNDPROC_GETNRDETECTEDDUNSETTINGS)(int , int *);
typedef int   (*PNDPROC_GETDETECTEDDUNSETTINGBYINDEX)(int , int, struct TomTom_DUNSetting *);

typedef int		(*PNDPROC_GETNRCOUNTRIES)(int );
typedef int		(*PNDPROC_GETCOUNTRYBYINDEX)(int , int , char* , int );
typedef int		(*PNDPROC_GETCOUNTRYNROFBASICDUNSETTINGS)(int , int );
typedef int		(*PNDPROC_GETCOUNTRYBASICDUNSETTINGBYINDEX)(int , int , int , struct TomTom_DUNSetting *);

typedef int		(*PNDPROC_GETNROFPHONES)(int );
typedef int		(*PNDPROC_GETPHONEBYINDEX)(int , int , int* , char* , int );

typedef int		(*PNDPROC_GETDUNSETTING)(int , int , int , struct TomTom_DUNSetting *);

class CApnd
{
public:
  CApnd();
  ~CApnd() {};
  bool LoadLib(char* DllFile);
  void UnloadLib();
  int Version();
  int Autodetect(char* mac, int chan, int gid);
  int Countries();
  int Phones();
  int Networks(int country);
  int Semi(int aPhoneIndex, int aBasicDunSettingId);
  void SetFile(FILE* aFp);
  bool Init(const char* aDbFile, bool aConnectToPhone);
  void DeInitialise();
  void AllNetworks();
private:
#ifdef EMULATE
public:
#endif
  void PrintDUNSetting(const TomTom_DUNSetting& DunSetting, const char* aPhoneName);
private:
  int iHandle; // handle of the engine
  FILE* iFp;   // file where output is directed to (stdout)

  //function pointers
  PNDPROC_INITIALISE	ifnInitialise;
  PNDPROC_DEINITIALISE	ifnDeInitialise;

  PNDPROC_GETVERSIONLIBRARY	ifnGetVersionLibrary;
  PNDPROC_GETVERSIONDATABASE	ifnGetVersionDatabase;

  PNDPROC_DETECTPHONEANDNETWORK	ifnDetectPhoneAndNetwork;
  PNDPROC_GETDETECTEDPHONE ifnGetDetectedPhone;
  PNDPROC_GETNRDETECTEDDUNSETTINGS ifnGetNrDetectedDunSettings;
  PNDPROC_GETDETECTEDDUNSETTINGBYINDEX ifnGetDetectedDunSettingByIndex;

  PNDPROC_GETNRCOUNTRIES ifnGetNrCountries;
  PNDPROC_GETCOUNTRYBYINDEX ifnGetCountryByIndex;
  
  PNDPROC_GETCOUNTRYNROFBASICDUNSETTINGS ifnGetCountryNrOfBasicDunSettings;
  PNDPROC_GETCOUNTRYBASICDUNSETTINGBYINDEX ifnGetCountryBasicDunSettingsByIndex;

  PNDPROC_GETNROFPHONES	ifnGetNrOfPhones;
  PNDPROC_GETPHONEBYINDEX	ifnGetPhoneByIndex;

  PNDPROC_GETDUNSETTING ifnGetDunSetting;

  TomTom_PNDPROC_RECVDATA	ifnRecvData;
  TomTom_PNDPROC_SENDDATA	ifnSendData;
  
#if defined(WINDOWS_PLATFORM)
	HINSTANCE	iNetDetectLibHandle;
#elif defined(LINUX_PLATFORM)  
  void* iNetDetectLibHandle;
#endif
};

#define ERROR_WRONGFUNCTION 102
#define ERROR_WRONGARGUMENTS 103
#define ERROR_NODLL 104
#define ERROR_BADOUTPUTFILE 105
#define ERROR_BADDLL 106
#define ERROR_BADDB 107
#define ERROR_NOCHAN 108

// globals
struct sigaction gSa;
BluetoothChannel gBTChan;
char* gFilename = NULL;

void SendTestATCommand();


#ifdef DEBUG_APND
  #define LS_DEBUG(msg...)     do {          printf(msg); fflush(stdout); }   while (0)
  #define LS_DEBUGT(t, msg...) do { if (t) { printf(msg); fflush(stdout); } } while (0)
#else
  #define LS_DEBUG(msg...)     do { } while (0)
  #define LS_DEBUGT(t, msg...) do { } while (0)
#endif


int	BTReceive(unsigned char* aData, int aSize, int aTimeOut)
{
//LS_DEBUG("BTReceive: ptr = %uld, timeout = %d, size = %d\n",aData, aTimeOut, aSize);
  
  int numbytes = gBTChan.ReceiveBinary((char*)aData, aSize, aTimeOut);

/*
  LS_DEBUG("BTReceive(%d):", numbytes);
  int i;
  for (i = 0; i < numbytes; i++)
  {
    LS_DEBUG("%.2x", aData[i]);
  }
  LS_DEBUG("\n");
  for (i = 0; i < numbytes; i++)
  {
    LS_DEBUG("%c", aData[i]);
  }
  LS_DEBUG("\n");
*/
  return numbytes;
}

int	BTSend(const unsigned char* aData, int aSize)
{
/*
  LS_DEBUG("BTSend:");
  int i;
  for (i  = 0; i < aSize; i++)
    LS_DEBUG("%.2x", aData[i]);
  LS_DEBUG("\n");
  for (i  = 0; i < aSize; i++)
    LS_DEBUG("%c", aData[i]);
  LS_DEBUG("\n");
*/
  return gBTChan.SendBinary(aData, aSize);
}


void SendTestATCommand(char* command)
{
  char* cmd = command;
  LS_DEBUG("Send [%s]\n",cmd);
  BTSend((unsigned char*)cmd, strlen(cmd));
  char buf[256];
  LS_DEBUG("Receive\n");
  int length = BTReceive((unsigned char*)buf, 256, 1000);
  if (length>=0)
    buf[length] = '\0';
  LS_DEBUG("Received: [%s]\n",buf);
}

void Clear(TomTom_DUNSetting* aDunSetting)
{
  memset(aDunSetting, 0, sizeof(TomTom_DUNSetting));
}

// handler function
void apnd_sig_term(int sig) 
{ 
  // remove output file
  if (gFilename)
    unlink(gFilename);
  // close connection to BlueAclRequestHandler
  gBTChan.Close();
  // exit with some error value
  exit(1);
}

// function to setup catching the signals, call this somewhere in top of your main
void apnd_catch_sigterm ()
{
  memset(&gSa, 0, sizeof(gSa));
  gSa.sa_flags = SA_NOCLDSTOP;
  gSa.sa_handler = apnd_sig_term;
  sigaction(SIGTERM, &gSa, NULL);
  sigaction(SIGINT,  &gSa, NULL);
}

int main(int argc, char* argv[])
{
  // Catch term signal
  apnd_catch_sigterm();

  CApnd Apnd;

  // Functions
  enum {NONE = -1, VERSION, AUTO, PHONES, COUNTRIES, NETWORKS, SEMI, REC, TRAP, ALL};
  int function = NONE;

  int channel = 0, GID = 0;
  int PhoneId = 0;
  int BasicDunSettingId = 0;
  int CountryId = 0;

//  for (int k = 0; k < argc; k++)
//    LS_DEBUG("%s\n", argv[k]);

  // Parse commandline
  if (argc < 3)
  {
    fprintf(stdout, "Not enough arguments\n");
    return ERROR_WRONGARGUMENTS;
  }

  if (!strcmp(argv[2], "version"))
  {
    if (argc != 3)  return ERROR_WRONGARGUMENTS;
    function = VERSION;
  }

  else if (!strcmp(argv[2], "auto"))
  {
    if (argc != 6)  return ERROR_WRONGARGUMENTS;
    function = AUTO;
    channel = Val((TChr*)argv[4]); 
    GID = Val((TChr*)argv[5]); 
  }
  else if (!strcmp(argv[2], "phones"))
  {
    if (argc != 3)  return ERROR_WRONGARGUMENTS;
    function = PHONES;
  }
  else if (!strcmp(argv[2], "countries"))
  {
    if (argc != 3)  return ERROR_WRONGARGUMENTS;
    function = COUNTRIES;
  }
  else if (!strcmp(argv[2], "network"))
  {
    if (argc != 4)  return ERROR_WRONGARGUMENTS;
    function = NETWORKS;
    CountryId = Val((TChr*)argv[3]);
  }
  else if (!strcmp(argv[2], "semi"))
  {
    if (argc != 5)  return ERROR_WRONGARGUMENTS;
    function = SEMI;
    PhoneId = Val((TChr*)argv[3]);
    BasicDunSettingId = Val((TChr*)argv[4]);
  }
#ifdef EMULATE
  else if (!strcmp(argv[2], "all"))
  {
    if (argc != 3)  return ERROR_WRONGARGUMENTS;
    function = ALL;
  }
  else if (!strcmp(argv[2], "receive"))
  {
    function = REC;
  }
  else if (!strcmp(argv[2], "trap"))
  {
    function = TRAP;
  }
#endif
  else
  {
    LS_DEBUG("Bad Command\n");
    return ERROR_WRONGFUNCTION;
  }

  int ret = PHONENETWORK_ERROR_NONE;
  FILE* fp;
  if (!strcmp(argv[1], "stdout"))
  {
    fp = stdout;
  }
  else
  {
    fp = fopen(argv[1], "w");
    gFilename = argv[1];
  }

  if (fp == NULL)
    return ERROR_BADOUTPUTFILE;

  Apnd.SetFile(fp);

#ifdef EMULATE
#define BUFSIZE 1024
  unsigned char buf[BUFSIZE];
  switch (function)
  {
  case VERSION:
    fprintf(fp, "666000\n");
    fprintf(fp, "999000\n");
    break;
  case  PHONES:
    fprintf(fp, "0,Ericsson P900\n");
    fprintf(fp, "1, Nokia 6230\n");
    fprintf(fp, "2,  Nokia 6600\n");    
    fprintf(fp, "3,   Nokia 8800\n");    
    fprintf(fp, "4,Siemens C60\n");   
    fprintf(fp, "5,Sony Ericsson K700i\n");
    fprintf(fp, "5,Sony Ericsson S700\n");
    break;
  case  COUNTRIES:
    fprintf(fp, "0,Belgium\n"); 
    fprintf(fp, "1,France\n"); 
    fprintf(fp, "2,Germany\n"); 
    fprintf(fp, "3,Netherlands\n"); 
    fprintf(fp, "4,Norway\n"); 
    fprintf(fp, "5,Portugal\n");
    fprintf(fp, "6,Spain\n");
    fprintf(fp, "7,Sweden\n");
    break;
  case  NETWORKS:
    fprintf(fp, "0,KPN\n"); 
    fprintf(fp, "1,Orange\n");
    fprintf(fp, "2,T-Mobile\n"); 
    fprintf(fp, "3,Vodafone\n");
    break;
  case AUTO:
  case  SEMI:
    TomTom_DUNSetting DunSetting;
	  strncpy(DunSetting.APN, "TestAPN", TomTom_PHONENETWORK_MAX_SIZE);
	  strncpy(DunSetting.ClientIP, "0.0.0.0", TomTom_PHONENETWORK_MAX_SIZE);
	  strncpy(DunSetting.Dns1, "0.0.0.0", TomTom_PHONENETWORK_MAX_SIZE);
	  strncpy(DunSetting.Dns2, "0.0.0.0", TomTom_PHONENETWORK_MAX_SIZE);
	  strncpy(DunSetting.ContractType, "", TomTom_PHONENETWORK_MAX_SIZE);
	  strncpy(DunSetting.Country, "Austria", TomTom_PHONENETWORK_MAX_SIZE);
	  strncpy(DunSetting.DefaultPassword, "password", TomTom_PHONENETWORK_MAX_SIZE);
	  strncpy(DunSetting.DefaultUser, "user", TomTom_PHONENETWORK_MAX_SIZE);
	  strncpy(DunSetting.DialString, "", TomTom_PHONENETWORK_MAX_SIZE);
	  strncpy(DunSetting.InitializationString, "", TomTom_PHONENETWORK_MAX_SIZE);
	  strncpy(DunSetting.DisplayName, "A1 (privat)", TomTom_PHONENETWORK_MAX_SIZE);
	  strncpy(DunSetting.GatewayIP, "0.0.0.0", TomTom_PHONENETWORK_MAX_SIZE);
	  strncpy(DunSetting.RequiresUser, "No", TomTom_PHONENETWORK_MAX_SIZE);
	  strncpy(DunSetting.DefaultSetting, "yes", TomTom_PHONENETWORK_MAX_SIZE);
	  DunSetting.aBasicDUNSettingId = 1;
    Apnd.PrintDUNSetting(DunSetting, "Siemens: C60");

    strncpy(DunSetting.APN, "TomTom Access", TomTom_PHONENETWORK_MAX_SIZE);
	  strncpy(DunSetting.ClientIP, "0.0.0.0", TomTom_PHONENETWORK_MAX_SIZE);
	  strncpy(DunSetting.Dns1, "0.0.0.0", TomTom_PHONENETWORK_MAX_SIZE);
	  strncpy(DunSetting.Dns2, "0.0.0.0", TomTom_PHONENETWORK_MAX_SIZE);
	  strncpy(DunSetting.ContractType, "expensive", TomTom_PHONENETWORK_MAX_SIZE);
	  strncpy(DunSetting.Country, "Netherlands", TomTom_PHONENETWORK_MAX_SIZE);
	  strncpy(DunSetting.DefaultPassword, "password", TomTom_PHONENETWORK_MAX_SIZE);
	  strncpy(DunSetting.DefaultUser, "user", TomTom_PHONENETWORK_MAX_SIZE);
	  strncpy(DunSetting.DialString, "ATDT0204469469", TomTom_PHONENETWORK_MAX_SIZE);
	  strncpy(DunSetting.InitializationString, "", TomTom_PHONENETWORK_MAX_SIZE);
	  strncpy(DunSetting.DisplayName, "A1 (privat)", TomTom_PHONENETWORK_MAX_SIZE);
	  strncpy(DunSetting.GatewayIP, "0.0.0.0", TomTom_PHONENETWORK_MAX_SIZE);
	  strncpy(DunSetting.RequiresUser, "No", TomTom_PHONENETWORK_MAX_SIZE);
	  strncpy(DunSetting.DefaultSetting, "yes", TomTom_PHONENETWORK_MAX_SIZE);
	  DunSetting.aBasicDUNSettingId = 2;
    Apnd.PrintDUNSetting(DunSetting, "");
    break;
  case REC:
  {
    LS_DEBUG("Init\n");
    gBTChan.Initialize("00:0E:6D:16:F0:A3", 1);
    LS_DEBUG("Open\n");
    gBTChan.Open();
    SendTestATCommand("AT\r\n");
    LS_DEBUG("Close\n");
    gBTChan.Close();
    break;
  }
  case TRAP:
    {
      time_t start;
      time(&start);
      time_t now = start;
      while (difftime(now, start) < 20)
      {
        for (int i = 0; i < 1000; i++)
          double x = sin(i);
        LS_DEBUG(".");
        fflush(stdout);
        time(&now);
      }
    }
  break;
  }

#else

  char* dbfile = "/mnt/sdcard/connect2internet.db";
  char* dllfile = "/mnt/sdcard/connect2internet.so";

  if (!Apnd.LoadLib(dllfile)) return ERROR_BADDLL;

  if (!Apnd.Init(dbfile, function == AUTO)) return ERROR_BADDB;

  switch (function)
  {
  case VERSION:
    ret = Apnd.Version();
    break;
  case AUTO:
    ret = Apnd.Autodetect(argv[3], channel, GID);
    break;
  case  PHONES:
    ret = Apnd.Phones();
    break;
  case  COUNTRIES:
    ret = Apnd.Countries();
    break;
  case  NETWORKS:
    ret = Apnd.Networks(CountryId);
    break;
  case  SEMI:
    ret = Apnd.Semi(PhoneId, BasicDunSettingId);
    break;
  case  ALL:
    Apnd.AllNetworks();
    break;
  }

  Apnd.DeInitialise();
#endif	

  if (fp != stdout)
  {
    int size = ftell(fp);
    LS_DEBUG("File size %d\n", size);
    fflush(fp);
    fsync(fileno(fp));
    fclose(fp);
    // remove empty files, i.e. if detection failed
    if (size == 0)
      unlink(gFilename);
  }
	return ret;
}

CApnd::CApnd()
{
  iHandle = -1;
  iFp = NULL;

  ifnInitialise = NULL;
  ifnDeInitialise = NULL;

  ifnGetVersionLibrary = NULL;
  ifnGetVersionDatabase = NULL;

  ifnDetectPhoneAndNetwork = NULL;
  ifnGetDetectedPhone = NULL;
  ifnGetNrDetectedDunSettings = NULL;
  ifnGetDetectedDunSettingByIndex = NULL;

  ifnGetNrCountries = NULL;
  ifnGetCountryByIndex = NULL;
  
  ifnGetCountryNrOfBasicDunSettings = NULL;
  ifnGetCountryBasicDunSettingsByIndex = NULL;

  ifnGetNrOfPhones = NULL;
  ifnGetPhoneByIndex = NULL;

  ifnGetDunSetting = NULL;

  ifnRecvData = NULL;
  ifnSendData = NULL;
  
	iNetDetectLibHandle = NULL;
}

void CApnd::SetFile(FILE* aFp)
{
  iFp = aFp;
}

bool CApnd::Init(const char* aDbFile, bool aConnectToPhone)
{
  if (aConnectToPhone)
  {
    ifnRecvData = BTReceive;
    ifnSendData = BTSend;
  }
  else
  {
    ifnRecvData = NULL;
	  ifnSendData = NULL;
  }
	iHandle = (*ifnInitialise)(aDbFile, ifnRecvData, ifnSendData);

  if (iHandle < 0 && iHandle > -10)
  {
		return false;
	}
  else
  {
    return true;
  }
}

void CApnd::DeInitialise()
{
  (*ifnDeInitialise)(iHandle);
  iHandle = 0;
  UnloadLib();
}

int CApnd::Version()
{  
  // print version info
	int ret = (*ifnGetVersionLibrary)(iHandle);
	fprintf(iFp, "%d\n", ret);

	ret = (*ifnGetVersionDatabase)(iHandle);
	fprintf(iFp, "%d\n", ret);
  return PHONENETWORK_ERROR_NONE;
} 

int CApnd::Autodetect(char* mac, int chan, int gid)
{
  // Open BT channel
  gBTChan.Initialize(mac, chan);
  system("killall -sigusr2 hcid");
  int ret = gBTChan.Open();
  system("killall -sigusr1 hcid");
  if (ret == -1)
    return ERROR_NOCHAN;

//SendTestATCommand("AT\r\n");
//SendTestATCommand("AT+CGMI\r\n");

  // Detection try a number of times to get around the BT timeout
  for (int i = 0; i < 3; i++)
  {
    ret = (ifnDetectPhoneAndNetwork)(iHandle);
    if (ret == PHONENETWORK_ERROR_NONE) 
      break;
    sleep(2);
  }
  
  if (ret != PHONENETWORK_ERROR_NONE) 
  {
    LS_DEBUG("DetectPhoneAndNetwork failed\n");
//    return ret;
  }

  // retreive phone
  char name[PHONENETWORK_MAX_SIZE];
  char manufact[PHONENETWORK_MAX_SIZE];
  char phonemodel[PHONENETWORK_MAX_SIZE];   // tomtom formatted version of the above
  int phoneid = (*ifnGetDetectedPhone)(iHandle, name, PHONENETWORK_MAX_SIZE, manufact, PHONENETWORK_MAX_SIZE);
  if (phoneid != -1)
  {
    sprintf(phonemodel, "%s: %s", manufact, name);
    LS_DEBUG("Phone: %s\n", phonemodel);
  }
  else
  {
    *phonemodel = 0;
    LS_DEBUG("GetDetectedPhone could not detected a phone\n");
  }

  // Retreive networks
  int count;
  ret = (*ifnGetNrDetectedDunSettings)(iHandle, &count);
  if (ret != PHONENETWORK_ERROR_NONE)
  {
    LS_DEBUG("ifnGetNrDetectedDunSettings failed\n");
    count = 0;  // beter be sure
  }

  TomTom_DUNSetting DunSetting;
  Clear(&DunSetting);

  LS_DEBUG("NrDetectedDunSettings = %d\n", count);
  if (count > 0)
  {
    for (int i = 0; i < count; i++)
    {
      ret = ifnGetDetectedDunSettingByIndex(iHandle, i, &DunSetting);
      if (ret == PHONENETWORK_ERROR_NONE)
        PrintDUNSetting(DunSetting, phonemodel);
      else
      {
        LS_DEBUG("GetDetectedDunSettingByIndex failed for %d\n", i);
        break;
      }
    }
  }
  else if (phoneid != -1)  // Only the phone was found
  {
//    ifnGetDunSetting(iHandle, phoneid, -1, &DunSetting);
    PrintDUNSetting(DunSetting, phonemodel);
  }

  LS_DEBUG("Closing channel\n");
  // Close BT channel
  gBTChan.Close();
  LS_DEBUG("Done\n");
  return ret;
}

void CApnd::PrintDUNSetting(const TomTom_DUNSetting& aDunSetting, const char* aPhoneName)
{
  LS_DEBUG("RequiresUser %s\n", aDunSetting.RequiresUser);
  LS_DEBUG("DefaultSetting %s\n", aDunSetting.DefaultSetting);
  int RequiresUser = 0;
	if (!strcmp(aDunSetting.RequiresUser,"Y")) RequiresUser = 1;

  int DefaultSetting = 0;
	if (!strcmp(aDunSetting.DefaultSetting,"Y")) DefaultSetting = 1;

  fprintf(iFp, "%s\n", aDunSetting.InitializationString);
	fprintf(iFp, "%s\n", aDunSetting.DialString);
	fprintf(iFp, "%s\n", aDunSetting.Dns1);
	fprintf(iFp, "%s\n", aDunSetting.Dns2);
	fprintf(iFp, "%s\n", aDunSetting.GatewayIP);
	fprintf(iFp, "%s\n", aDunSetting.APN);
	fprintf(iFp, "%s\n", aDunSetting.DisplayName); 
	fprintf(iFp, "%s\n", aDunSetting.ClientIP);
	fprintf(iFp, "%s\n", aDunSetting.ContractType);
	fprintf(iFp, "%d\n", RequiresUser);
	fprintf(iFp, "%s\n", aDunSetting.DefaultUser);
	fprintf(iFp, "%s\n", aDunSetting.DefaultPassword);
	fprintf(iFp, "%d\n", DefaultSetting);
	fprintf(iFp, "%s\n", aDunSetting.Country);
	//fprintf(iFp, "%d\n", aDunSetting.aBasicDUNSettingId);
  if (aPhoneName)
	  fprintf(iFp, "%s\n", aPhoneName);
  else
	  fprintf(iFp, "\n");
}

int CApnd::Countries() 
{
  int ret;
	int nrcountries = (*ifnGetNrCountries)(iHandle);
	char	buf[PHONENETWORK_MAX_SIZE];
	for (int i = 0; i < nrcountries; i++)
	{
		ret = (*ifnGetCountryByIndex)(iHandle, i, buf, PHONENETWORK_MAX_SIZE);
		if (ret == PHONENETWORK_ERROR_NONE)
			fprintf(iFp, "%d,%s\n", i, buf);
    else
      break;
	}
  return ret;
}

int CApnd::Phones()
{
  int ret;

	int nrphones = (*ifnGetNrOfPhones)(iHandle);
	for (int i = 0; i < nrphones;i ++)
	{
  	char	buf[PHONENETWORK_MAX_SIZE];
    int id;
		ret = (*ifnGetPhoneByIndex)(iHandle, i, &id, buf, PHONENETWORK_MAX_SIZE);
		if (ret == PHONENETWORK_ERROR_NONE)
			fprintf(iFp, "%d,%s\n", id, buf);
    else 
      break;
	}
  return ret;
}

int CApnd::Networks(int aCountryIndex) 
{
  int ret;
  int Count = (*ifnGetCountryNrOfBasicDunSettings)(iHandle, aCountryIndex);
  LS_DEBUG("nrnetworks for country %d %d\n", aCountryIndex, Count);
  for (int i = 0; i < Count; i++)
  {
    TomTom_DUNSetting DunSetting;
    Clear(&DunSetting);
	  ret=(*ifnGetCountryBasicDunSettingsByIndex)(iHandle, aCountryIndex, i, &DunSetting);
    if (ret==TomTom_PHONENETWORK_ERROR_NONE)
    {
			fprintf(iFp, "%d,%s\n", 
        DunSetting.aBasicDUNSettingId,
        DunSetting.DisplayName);
    }
   }
  return ret;
}


int CApnd::Semi(int aPhoneIndex, int aBasicDunSettingId)
{
  TomTom_DUNSetting DunSetting;
  Clear(&DunSetting);
  LS_DEBUG("Semi %d %d \n", aPhoneIndex, aBasicDunSettingId);
  int ret = (*ifnGetDunSetting)(iHandle, aPhoneIndex, aBasicDunSettingId, &DunSetting);
	if (ret == PHONENETWORK_ERROR_NONE)
  {
    PrintDUNSetting(DunSetting, NULL);
  }
  return ret;
}

void CApnd::AllNetworks() 
{
  int ret;
	int nrcountries = (*ifnGetNrCountries)(iHandle);
	char	buf[PHONENETWORK_MAX_SIZE];
  TomTom_DUNSetting DunSetting;
  Clear(&DunSetting);
	for (int country = 0; country < nrcountries; country++)
	{
		ret = (*ifnGetCountryByIndex)(iHandle, country, buf, PHONENETWORK_MAX_SIZE);
		if (ret == PHONENETWORK_ERROR_NONE)
			fprintf(iFp, "%d,%s\n", country, buf);
    else
    {
			fprintf(iFp, "Cannot get data for country %d\n", country);
      break;
    }
    int nrnetworks = (*ifnGetCountryNrOfBasicDunSettings)(iHandle, country);
		fprintf(iFp, "NrNetworks = %d\n", nrnetworks);
    for (int i = 0; i < nrnetworks; i++)
    {
      int ret = (*ifnGetCountryBasicDunSettingsByIndex)(iHandle, country, i, &DunSetting);
			fprintf(iFp, "network %d ret = %d, %d,%s %s\n", 
        i,
        ret,
        DunSetting.aBasicDUNSettingId,
        DunSetting.DisplayName,
        DunSetting.Country);
    }
	}
}

#if defined(WINDOWS_PLATFORM)
bool CApnd::LoadLib(char* DllFile)
{
	iNetDetectLibHandle=::LoadLibrary(DllFile);
	if (iNetDetectLibHandle==NULL)
	{
		return false;
	}

	ifnInitialise=(PNDPROC_INITIALISE)::GetProcAddress(iNetDetectLibHandle,"TomTom_Initialise");
	ifnDeInitialise=(PNDPROC_DEINITIALISE)::GetProcAddress(iNetDetectLibHandle,"TomTom_DeInitialise");
	ifnGetVersionLibrary=(PNDPROC_GETVERSIONLIBRARY)::GetProcAddress(iNetDetectLibHandle,"TomTom_GetVersionLibrary");
	ifnGetVersionDatabase=(PNDPROC_GETVERSIONDATABASE)::GetProcAddress(iNetDetectLibHandle,"TomTom_GetVersionDatabase");
	ifnDetectPhoneAndNetwork=(PNDPROC_DETECTPHONEANDNETWORK)::GetProcAddress(iNetDetectLibHandle,"TomTom_DetectPhoneAndNetwork");
	ifnGetNrCountries=(PNDPROC_GETNRCOUNTRIES)::GetProcAddress(iNetDetectLibHandle,"TomTom_GetNrCountries");
	ifnGetCountryByIndex=(PNDPROC_GETCOUNTRYBYINDEX)::GetProcAddress(iNetDetectLibHandle,"TomTom_GetCountryByIndex");
	ifnGetNrOfPhones=(PNDPROC_GETNROFPHONES)::GetProcAddress(iNetDetectLibHandle,"TomTom_GetNrOfPhones");
	ifnGetPhoneByIndex=(PNDPROC_GETPHONEBYINDEX)::GetProcAddress(iNetDetectLibHandle,"TomTom_GetPhoneByIndex");
  ifnGetCountryNrOfBasicDunSettings=(PNDPROC_GETCOUNTRYNROFBASICDUNSETTINGS)::GetProcAddress(iNetDetectLibHandle,"TomTom_GetCountryNrOfBasicDUNSettings");
  ifnGetCountryBasicDunSettingsByIndex=(PNDPROC_GETCOUNTRYBASICDUNSETTINGBYINDEX)::GetProcAddress(iNetDetectLibHandle,"TomTom_GetCountryBasicDUNSettingsbyIndex");
  ifnGetDetectedPhone=(PNDPROC_GETDETECTEDPHONE)::GetProcAddress(iNetDetectLibHandle,"TomTom_GetDetectedPhone");
  ifnGetNrDetectedDunSettings=(PNDPROC_GETNRDETECTEDDUNSETTINGS)::GetProcAddress(iNetDetectLibHandle,"TomTom_GetNrDetectedDUNSettings");
  ifnGetDetectedDunSettingByIndex=(PNDPROC_GETDETECTEDDUNSETTINGBYINDEX)::GetProcAddress(iNetDetectLibHandle,"TomTom_GetDetectedDUNSettingByIndex");
  ifnGetDunSetting=(PNDPROC_GETDUNSETTING)::GetProcAddress(iNetDetectLibHandle,"TomTom_GetDUNSetting");
  
	if (!(ifnInitialise &&
	  ifnDeInitialise &&
	  ifnGetVersionLibrary &&
	  ifnGetVersionDatabase &&
	  ifnDetectPhoneAndNetwork &&
	  ifnGetNrCountries &&
	  ifnGetCountryByIndex &&
	  ifnGetNrOfPhones &&
	  ifnGetPhoneByIndex &&
    ifnGetCountryNrOfBasicDunSettings &&
    ifnGetCountryBasicDunSettingsByIndex &&
    ifnDetectPhoneAndNetwork &&
    ifnGetDetectedPhone &&
    ifnGetNrDetectedDunSettings &&
    ifnGetDetectedDunSettingByIndex))
	{
    UnloadLib();
    return false;
	}
  return true;
}

void CApnd::UnloadLib()
{
	::FreeLibrary(iNetDetectLibHandle);
	iNetDetectLibHandle=NULL;
}

#elif defined(LINUX_PLATFORM)  

bool CApnd::LoadLib(char* DllFile)
{
  // Open the shared library
  iNetDetectLibHandle = dlopen (DllFile, RTLD_LAZY);
  
  // Check to see if there were any errors in opening the library.
  char* error;
  if ((error = dlerror()) != NULL)
  {
    iNetDetectLibHandle = NULL;
    fprintf(stderr,"%s\n",error);
    return false;
  }
  
  // Get the functions out of the library.
  ifnInitialise = (PNDPROC_INITIALISE)dlsym (iNetDetectLibHandle, "TomTom_Initialise");
  if ((error = dlerror()) != NULL)
  {
    fprintf(stderr,"%s\n",error);
    ifnInitialise = NULL;
  }
  ifnDeInitialise = (PNDPROC_DEINITIALISE)dlsym (iNetDetectLibHandle, "TomTom_DeInitialise");
  if ((error = dlerror()) != NULL)
  {
    fprintf(stderr,"%s\n",error);
    ifnDeInitialise = NULL;
  }
  ifnGetVersionLibrary = (PNDPROC_GETVERSIONLIBRARY)dlsym (iNetDetectLibHandle, "TomTom_GetVersionLibrary");
  if ((error = dlerror()) != NULL)
  {
    fprintf(stderr,"%s\n",error);
    ifnGetVersionLibrary = NULL;
  }
  ifnGetVersionDatabase = (PNDPROC_GETVERSIONDATABASE)dlsym (iNetDetectLibHandle, "TomTom_GetVersionDatabase");
  if ((error = dlerror()) != NULL)
  {
    fprintf(stderr,"%s\n",error);
    ifnGetVersionDatabase = NULL;
  }
  ifnDetectPhoneAndNetwork = (PNDPROC_DETECTPHONEANDNETWORK)dlsym (iNetDetectLibHandle, "TomTom_DetectPhoneAndNetwork");
  if ((error = dlerror()) != NULL)
  {
    fprintf(stderr,"%s\n",error);
    ifnDetectPhoneAndNetwork = NULL;
  }
  ifnGetNrCountries = (PNDPROC_GETNRCOUNTRIES)dlsym (iNetDetectLibHandle, "TomTom_GetNrCountries");
  if ((error = dlerror()) != NULL)
  {
    fprintf(stderr,"%s\n",error);
    ifnGetNrCountries = NULL;
  }
  ifnGetCountryByIndex = (PNDPROC_GETCOUNTRYBYINDEX)dlsym (iNetDetectLibHandle, "TomTom_GetCountryByIndex");
  if ((error = dlerror()) != NULL)
  {
    fprintf(stderr,"%s\n",error);
    ifnGetCountryByIndex = NULL;
  }
  ifnGetNrOfPhones = (PNDPROC_GETNROFPHONES)dlsym (iNetDetectLibHandle, "TomTom_GetNrOfPhones");
  if ((error = dlerror()) != NULL)
  {
    fprintf(stderr,"%s\n",error);
    ifnGetNrOfPhones = NULL;
  }
  ifnGetPhoneByIndex = (PNDPROC_GETPHONEBYINDEX)dlsym (iNetDetectLibHandle, "TomTom_GetPhoneByIndex");
  if ((error = dlerror()) != NULL)
  {
    fprintf(stderr,"%s\n",error);
    ifnGetPhoneByIndex = NULL;
  }
  ifnGetCountryNrOfBasicDunSettings = (PNDPROC_GETCOUNTRYNROFBASICDUNSETTINGS)dlsym (iNetDetectLibHandle, "TomTom_GetCountryNrOfBasicDUNSettings");
  if ((error = dlerror()) != NULL)
  {
    fprintf(stderr,"%s\n",error);
    ifnGetCountryNrOfBasicDunSettings = NULL;
  }
  ifnGetCountryBasicDunSettingsByIndex = (PNDPROC_GETCOUNTRYBASICDUNSETTINGBYINDEX)dlsym (iNetDetectLibHandle, "TomTom_GetCountryBasicDUNSettingByIndex");
  if ((error = dlerror()) != NULL)
  {
    fprintf(stderr,"%s\n",error);
    ifnGetCountryBasicDunSettingsByIndex = NULL;
  }
  ifnGetDetectedPhone = (PNDPROC_GETDETECTEDPHONE)dlsym (iNetDetectLibHandle, "TomTom_GetDetectedPhone");
  if ((error = dlerror()) != NULL)
  {
    fprintf(stderr,"%s\n",error);
    ifnGetDetectedPhone = NULL;
  }
  ifnGetNrDetectedDunSettings = (PNDPROC_GETNRDETECTEDDUNSETTINGS)dlsym (iNetDetectLibHandle, "TomTom_GetNrDetectedDUNSettings");
  if ((error = dlerror()) != NULL)
  {
    fprintf(stderr,"%s\n",error);
    ifnGetNrDetectedDunSettings = NULL;
  }
  ifnGetDetectedDunSettingByIndex = (PNDPROC_GETDETECTEDDUNSETTINGBYINDEX)dlsym (iNetDetectLibHandle, "TomTom_GetDetectedDUNSettingByIndex");
  if ((error = dlerror()) != NULL)
  {
    fprintf(stderr,"%s\n",error);
    ifnGetDetectedDunSettingByIndex = NULL;
  }
  ifnGetDunSetting = (PNDPROC_GETCOUNTRYBASICDUNSETTINGBYINDEX)dlsym (iNetDetectLibHandle, "TomTom_GetDUNSetting");
  if ((error = dlerror()) != NULL)
  {
    fprintf(stderr,"%s\n",error);
    ifnGetDunSetting = NULL;
  }
  
  if (!(ifnInitialise &&
    ifnDeInitialise &&
    ifnGetVersionLibrary &&
    ifnGetVersionDatabase &&
    ifnDetectPhoneAndNetwork &&
    ifnGetNrCountries &&
    ifnGetCountryByIndex &&
    ifnGetNrOfPhones &&
    ifnGetPhoneByIndex &&
    ifnGetCountryNrOfBasicDunSettings &&
    ifnGetCountryBasicDunSettingsByIndex &&
    ifnGetDetectedPhone &&
    ifnGetNrDetectedDunSettings &&
    ifnGetDetectedDunSettingByIndex &&
    ifnGetDunSetting))
  {
    LS_DEBUG("Error in getting proc addresses\n");
    if (!(ifnInitialise)) LS_DEBUG("ifnInitialise\n");
    if (!(ifnDeInitialise)) LS_DEBUG("ifnDeInitialise\n");
    if (!(ifnGetVersionLibrary)) LS_DEBUG("ifnGetVersionLibrary\n");
    if (!(ifnGetVersionDatabase)) LS_DEBUG("ifnGetVersionDatabase\n");
    if (!(ifnDetectPhoneAndNetwork)) LS_DEBUG("ifnDetectPhoneAndNetwork\n");
    if (!(ifnGetNrCountries)) LS_DEBUG("ifnGetNrCountries\n");
    if (!(ifnGetCountryByIndex)) LS_DEBUG("ifnGetCountryByIndex\n");
    if (!(ifnGetNrOfPhones)) LS_DEBUG("ifnGetNrOfPhones\n");
    if (!(ifnGetPhoneByIndex)) LS_DEBUG("ifnGetPhoneByIndex\n");
    if (!(ifnGetCountryNrOfBasicDunSettings)) LS_DEBUG("ifnGetCountryNrOfBasicDunSettings\n");
    if (!(ifnGetCountryBasicDunSettingsByIndex)) LS_DEBUG("ifnGetCountryBasicDunSettingsByIndex\n");
    if (!(ifnDetectPhoneAndNetwork)) LS_DEBUG("ifnDetectPhoneAndNetwork\n");
    if (!(ifnGetDetectedPhone)) LS_DEBUG("ifnGetDetectedPhone\n");
    if (!(ifnGetNrDetectedDunSettings)) LS_DEBUG("ifnGetNrDetectedDunSettings\n");
    if (!(ifnGetDetectedDunSettingByIndex)) LS_DEBUG("ifnGetDetectedDunSettingByIndex\n");
    
    UnloadLib();
    return false;
  }
  return true;
}

void CApnd::UnloadLib()
{
  dlclose(iNetDetectLibHandle);
  iNetDetectLibHandle = NULL;
}

#else  // NO Platform

bool CApnd::LoadLib(char* DllFile)
{
  return false;
}

void CApnd::UnloadLib()
{
}

#endif
